import random
import pickle

# 手を表す定数
ROCK = 0
SCISSORS = 1
PAPER = 2
HAND = [ROCK, SCISSORS, PAPER]

# 勝敗を表す定数
DRAW = 0
USER_LOST = 1
USER_WIN = 2
USER_JUDGE = ["あいこ", "ユーザーの負け", "ユーザーの勝ち"]

# ジャンケンを1回行い、勝敗とユーザーが出した手を返す関数
def play(best_computer_hand=None):
    # ユーザーが手を選ぶ
    while True:
        # 手をキー入力する
        print("******* 手を選んでください *******")
        print(f"グー = {ROCK}, チョキ = {SCISSORS}, パー = {PAPER}")
        user_hand = input("ユーザーの手:")
        # 数字が入力されたことをチェックする
        if user_hand.isnumeric():
            user_hand = int(user_hand)
        else:
            continue
        # 手を表す数値であることをチェックする
        if user_hand in HAND:
            break

    # コンピュータが手を選ぶ
    if best_computer_hand is None:
        # 乱数で手を選ぶ
        computer_hand = random.choice(HAND)
    else:
        # 外部から引数で指定された手を設定する
        computer_hand = best_computer_hand
    # コンピュータが選んだ手を表示する
    print(f"コンピュータの手:{computer_hand}")

    # 勝敗を判定する
    judge = (user_hand - computer_hand + 3) % 3
    print(USER_JUDGE[judge])
    print()

    # 勝敗とユーザーが出した手を戻り値として返す
    return judge, user_hand

# ユーザーの癖の記録
# それぞれの手を出した合計回数（図2）
total_user_hand = [0, 0, 0]
# 直前があいこのとき次に出す手の回数（図3の1）
after_draw_user_hand = [[0, 0, 0], [0, 0, 0], [0, 0, 0]]
# 直前が負けのとき次に出す手の回数（図3の2）
after_lost_user_hand = [[0, 0, 0], [0, 0, 0], [0, 0, 0]]
# 直前が勝ちのとき次に出す手の回数（図3の3）
after_win_user_hand = [[0, 0, 0], [0, 0, 0], [0, 0, 0]]

# num回のジャンケンを行い、ユーザーの癖を記録する関数
def check_habit(num):
    # ユーザーの勝ちの回数
    user_win_num = 0
    # 直前の勝敗
    last_judge = None
    # 直前のユーザーの手
    last_user_hand = None

    # num回のジャンケンを行い、ユーザーの癖を記録する
    for n in range(num):
        while True:
            # 現在の回数を表示する
            print(f"【第{n + 1}回戦】")
            # ジャンケンを1回行い、現在の勝敗とユーザーの手を得る
            current_judge, current_user_hand = play(None)
            # ユーザーの勝ちの回数をカウントする
            if current_judge == USER_WIN:
                user_win_num += 1

            # それぞれの手を出した合計回数をカウントする
            total_user_hand[current_user_hand] += 1
            # 直前があいこのとき次に出す手の回数をカウントする
            if last_judge == DRAW:
                after_draw_user_hand[last_user_hand][current_user_hand] += 1
            # 直前が負けのとき次に出す手の回数をカウントする
            elif last_judge == USER_LOST:
                after_lost_user_hand[last_user_hand][current_user_hand] += 1
            # 直前が勝ちのとき次に出す手の回数をカウントする
            elif last_judge == USER_WIN:
                after_win_user_hand[last_user_hand][current_user_hand] += 1

            # 現在の勝敗とユーザーの手を、直前の勝敗とユーザーの手に設定する
            last_judge = current_judge;
            last_user_hand = current_user_hand;

            # あいこでないなら次の勝負に進む
            if current_judge != DRAW:
                break

    # ユーザーの勝ちの回数を戻り値として返す
    return user_win_num

# ユーザーの癖の記録を表示する関数
def display_habit():
    print("【ユーザーの癖の記録】")
    print("それぞれの手を出した回数:" + str(total_user_hand))
    print("直前があいこのとき次に出す手の回数:" + str(after_draw_user_hand))
    print("直前が負けのとき次に出す手の回数:" + str(after_lost_user_hand))
    print("直前が勝ちのとき次に出す手の回数:" + str(after_win_user_hand))

# ユーザーの癖の記録をfile_nameファイルに書き込む関数
def write_habit(file_name):
    # 記録をファイルに書き込む
    with open(file_name, mode="wb") as file_obj:
        pickle.dump(total_user_hand, file_obj)
        pickle.dump(after_draw_user_hand, file_obj)
        pickle.dump(after_lost_user_hand, file_obj)
        pickle.dump(after_win_user_hand, file_obj)

# ユーザーの癖をfile_nameファイルから読み出す関数
def read_habit(file_name):
    global total_user_hand,\
           after_draw_user_hand, after_lost_user_hand, after_win_user_hand
    # 記録をファイルから読み出す
    with open(file_name, mode="rb") as file_obj:
        total_user_hand = pickle.load(file_obj)
        after_draw_user_hand = pickle.load(file_obj)
        after_lost_user_hand = pickle.load(file_obj)
        after_win_user_hand = pickle.load(file_obj)

# メインプログラム
if __name__ == "__main__":
    # ジャンケンの回数を設定する
    num = 50

    # ジャンケンしてユーザーの癖を記録する
    user_win = check_habit(num)
    print(f"【ユーザーの対戦結果】{num}戦:{user_win}勝{num - user_win}敗")
    print()

    # ユーザーの癖をファイルに書き込む
    write_habit("myhabit.dat")
    print("******* ファイルに書き込んだ記録 *******")
    display_habit()
    print()

    # ファイルからユーザーの癖を読み出す
    read_habit("myhabit.dat")
    print("******* ファイルから読み出した記録 *******")
    display_habit()
